"use strict";(self.webpackChunkd3vis_ipynb=self.webpackChunkd3vis_ipynb||[]).push([[493],{493:(e,o,i)=>{i.r(o),i.d(o,{default:()=>a,helloWidgetPlugin:()=>d});var t=i(801),l=i(420);const d={id:"d3vis_ipynb:plugin",requires:[t.IJupyterWidgetRegistry],activate:function(e,o){o.registerWidget({name:"d3vis_ipynb",version:l.version,exports:{BarPlotModel:l.BarPlotModel,BarPlotView:l.BarPlotView,ButtonModel:l.ButtonModel,ButtonView:l.ButtonView,DecisionPlotModel:l.DecisionPlotModel,DecisionPlotView:l.DecisionPlotView,DropdownModel:l.DropdownModel,DropdownView:l.DropdownView,HistogramPlotModel:l.HistogramPlotModel,HistogramPlotView:l.HistogramPlotView,ImageModel:l.ImageModel,ImageView:l.ImageView,InputModel:l.InputModel,InputView:l.InputView,LinearPlotModel:l.LinearPlotModel,LinearPlotView:l.LinearPlotView,MatrixLayoutModel:l.MatrixLayoutModel,MatrixLayoutView:l.MatrixLayoutView,RangeSliderModel:l.RangeSliderModel,RangeSliderView:l.RangeSliderView,RidgelinePlotModel:l.RidgelinePlotModel,RidgelinePlotView:l.RidgelinePlotView,ScatterPlotModel:l.ScatterPlotModel,ScatterPlotView:l.ScatterPlotView,TextAreaModel:l.TextAreaModel,TextAreaView:l.TextAreaView,TextModel:l.TextModel,TextView:l.TextView,VideoModel:l.VideoModel,VideoView:l.VideoView,WaterfallPlotModel:l.WaterfallPlotModel,WaterfallPlotView:l.WaterfallPlotView}})},autoStart:!0},a=d}}]);