.graph {
  border: 1px solid #ccc !important;
}

/***** EMBEDDING *****/
.basic .dashboard-div {
  border: 1px solid black;
  text-align: center;
}

.dark .dashboard-div {
  margin: 2px;
  border-radius: 10px;
  background-color: #262626;
  box-shadow: 0 4px 8px rgba(0, 0, 0, 0.5);
}

.glassmorphism .dashboard-div {
  margin: 5px;
  border-radius: 10px;
  background: rgb(255, 254, 254);
  border: 1px solid rgb(255, 255, 255);
  backdrop-filter: blur(10px);
  box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);
}

.neumorphism .dashboard-div {
  margin: 10px;
  border-radius: 15px;
  background-color: #e0e0e0;
  box-shadow: 5px 5px 10px #bebebe, -5px -5px 10px #ffffff;
}

.minimalism .dashboard-div {
  margin-bottom: 5px;
}

/***** RANGE SLIDER *****/
.range_outside_container {
  display: flex;
  max-width: 500px;
}

.range_inside_container {
  display: flex;
  flex-direction: column;
  width: 80%;
}

.range_description {
  padding-right: 20px;
}

.range_value {
  padding-left: 20px;
  width: 80px;
}

.sliders_control {
  position: relative;
  min-height: 50px;
}

input[type="range"]::-webkit-slider-thumb {
  -webkit-appearance: none;
  pointer-events: all;
  width: 24px;
  height: 24px;
  background-color: #fff;
  border-radius: 50%;
  box-shadow: 0 0 0 1px #c6c6c6;
  cursor: pointer;
}

input[type="range"]::-moz-range-thumb {
  -webkit-appearance: none;
  pointer-events: all;
  width: 24px;
  height: 24px;
  background-color: #fff;
  border-radius: 50%;
  box-shadow: 0 0 0 1px #c6c6c6;
  cursor: pointer;
}

input[type="range"]::-webkit-slider-thumb:hover {
  background: #f7f7f7;
}

input[type="range"]::-webkit-slider-thumb:active {
  box-shadow: inset 0 0 3px #387bbe, 0 0 9px #387bbe;
  -webkit-box-shadow: inset 0 0 3px #387bbe, 0 0 9px #387bbe;
}

input[type="range"] {
  -webkit-appearance: none;
  appearance: none;
  height: 10px;
  width: 100%;
  border: none !important;
  position: absolute;
  background-color: #f79847;
  pointer-events: none;
}

.top_slider {
  height: 0 !important;
  z-index: 1;
  top: 5px;
}
